@@ -24,13 +24,13 @@
 #include <grp.h>
 #include <libgen.h>
 #include <fcntl.h>
+#include <sys/time.h>
 
 #if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__APPLE__) || defined(__FreeBSD_kernel__)
 #include <sys/param.h>
 #include <sys/mount.h>
 #include <sys/socket.h>
 #include <sys/sysctl.h>
-#include <sys/time.h>
 #include <net/if.h>
 #include <ifaddrs.h>
 #endif
@@ -42,6 +42,15 @@
 #define DECCONV
 #endif
 
+/* used in debug to get function name */
+#if __STDC_VERSION__ < 199901L
+#if __GNUC__ >= 2
+#define __func__ __FUNCTION__
+#else
+#define __func__ "function"
+#endif
+#endif
+
 /*
 
 Note! These are only the default values for settings
@@ -57,17 +66,24 @@ and most can be changed later from the config file.
 #define DATABASEDIR "/var/lib/vnstat"
 #endif
 #endif
+
+/* database file name */
 #define DATABASEFILE "vnstat.db"
 
 /* on which day should months change */
 #define MONTHROTATE 1
+#define MONTHROTATEYEARS 0
 
 /* date output formats for -d, -m, -t and image header*/
-/* see 'man date' for control codes       <1.8 values */
-#define DFORMAT "%x"                    /* "%d.%m." */
-#define MFORMAT "%b '%y"                /* "%b '%y" */
-#define TFORMAT "%x"                    /* "%d.%m.%y" */
-#define HFORMAT "%x %H:%M"              /* "%d.%m.%Y %H:%M" */
+/* see 'man date' for control codes      1.x values     <1.8 values */
+#define DFORMAT "%Y-%m-%d"		 /* "%x"         "%d.%m." */
+#define MFORMAT "%Y-%m"			 /* "%b '%y"     "%b '%y" */
+#define TFORMAT "%Y-%m-%d"		 /* "%x"         "%d.%m.%y" */
+#define HFORMAT "%Y-%m-%d %H:%M" /* "%x %H:%M"   "%d.%m.%Y %H:%M" */
+
+#ifndef DATETIMEFORMAT
+#define DATETIMEFORMAT "%Y-%m-%d %H:%M:%S"
+#endif
 
 /* characters used for visuals */
 #define RXCHAR "%"
@@ -85,16 +101,13 @@ and most can be changed later from the config file.
 
 /* output style */
 /* 0 = minimal/narrow, 1 = bars everywhere */
-/* 2 = same as 1 + rate in summary and weekly */
+/* 2 = same as 1 + rate in summary */
 /* 3 = rate everywhere */
 #define OSTYLE 3
 
 /* rate in vnstati summary output */
 #define SUMMARYRATE 1
 
-/* layout of summary output in vnstati */
-#define SUMMARYLAYOUT 1
-
 /* rate in vnstati hourly output */
 #define HOURLYRATE 1
 
@@ -131,16 +144,9 @@ and most can be changed later from the config file.
 /* how many seconds should sampling take by default */
 #define DEFSAMPTIME 5
 
-/* maximum time (minutes) between two updates before traffic */
-/* for that period will be discarded */
-/* set to a little over one hour so that it doesn't break using */
-/* cron.hourly like Gentoo seems to do */
-#define MAXUPDATEINTERVAL 62
-
 /* default query mode */
-/* 0 = normal, 1 = days, 2 = months, 3 = top10 */
-/* 4 = exportdb, 5 = short, 6 = weeks, 7 = hours */
-/* 8 = xml */
+/* 0 = normal, 1 = days, 2 = months, 3 = top, 5 = short */
+/* 7 = hours, 8 = xml, 9 = one line, 10 = json */
 #define DEFQMODE 0
 
 /* how much the boot time can variate between updates (seconds) */
@@ -149,30 +155,39 @@ and most can be changed later from the config file.
 /* check disk space by default */
 #define USESPACECHECK 1
 
-/* use file locking by default */
-#define USEFLOCK 1
+/* create trafficless entries by default */
+#define TRAFFICLESSENTRIES 1
 
-/* log trafficless days by default */
-#define TRAFLESSDAY 1
+/* list outputs */
+#define LISTFIVEMINS 24
+#define LISTHOURS 24
+#define LISTDAYS 30
+#define LISTMONTHS 12
+#define LISTYEARS 0
+#define LISTTOP 10
+#define LISTJSONXML 0
+
+/* data retention defaults */
+#define FIVEMINUTEHOURS 48
+#define HOURLYDAYS 4
+#define DAILYDAYS 62
+#define MONTHLYMONTHS 25
+#define YEARLYYEARS -1
+#define TOPDAYENTRIES 20
 
 /* assume that locale can be UTF-n when enabled */
 #define UTFLOCALE 1
 
-/* how many times try file locking before giving up */
-/* each try takes about a second */
-#define LOCKTRYLIMIT 5
-
-/* database version */
-/* 1 = 1.0, 2 = 1.1-1.2, 3 = 1.3- */
-#define DBVERSION 3
+/* 1 = 2.0 */
+#define SQLDBVERSION "1"
 
 /* xml format version */
-/* 1 = 1.7- */
-#define XMLVERSION 1
+/* 1 = 1.7-1.16, 2 = 2.0 */
+#define XMLVERSION 2
 
 /* json format version */
-/* 1 = 1.13- */
-#define JSONVERSION 1
+/* 1 = 1.13-1.16, 2 = 2.0 */
+#define JSONVERSION 2
 
 /* json format version, -tr */
 /* 1 = 1.18- */
@@ -204,7 +219,7 @@ and most can be changed later from the config file.
 #endif
 
 /* daemon defaults */
-#define UPDATEINTERVAL 30
+#define UPDATEINTERVAL 20
 #define TIMESYNCWAIT 5
 #define POLLINTERVAL 5
 #define SAVEINTERVAL 5
@@ -215,6 +230,7 @@ and most can be changed later from the config file.
 #define UPDATEFILEOWNER 1
 #define LOGFILE "/var/log/vnstat/vnstat.log"
 #define PIDFILE "/var/run/vnstat/vnstat.pid"
+#define IS64BIT -2
 
 /* no transparency by default */
 #define TRANSBG 0
@@ -233,6 +249,9 @@ and most can be changed later from the config file.
 #define CTX "606060"
 #define CTXD "-"
 
+/* number of retries for non-fatal database errors */
+#define DBRETRYLIMIT 3
+
 /* internal config structure */
 typedef struct {
 	char dformat[64], mformat[64], tformat[64], hformat[64];
@@ -243,59 +262,28 @@ typedef struct {
 	char cbg[8], cedge[8], cheader[8], cheadertitle[8], cheaderdate[8], ctext[8];
 	char cline[8], clinel[8], cvnstat[8], crx[8], crxd[8], ctx[8], ctxd[8];
 	int32_t unitmode, rateunitmode, rateunit, bvar, qmode, sampletime, hourlyrate, summaryrate;
-	int32_t monthrotate, maxbw, flock, spacecheck, traflessday, transbg, slayout, ostyle;
-	int32_t defaultdecimals, hourlydecimals, hourlystyle;
+	int32_t monthrotate, monthrotateyears, maxbw, spacecheck, trafficlessentries, transbg, ostyle;
+	int32_t defaultdecimals, hourlydecimals, hourlystyle, is64bit;
 	char cfgfile[512], logfile[512], pidfile[512];
 	char daemonuser[33], daemongroup[33];
 	int32_t timesyncwait, updateinterval, pollinterval, saveinterval, offsaveinterval, savestatus;
 	int32_t uselogging, createdirs, updatefileowner, bwdetection, bwdetectioninterval, utflocale;
+	int32_t fiveminutehours, hourlydays, dailydays, monthlymonths, yearlyyears, topdayentries;
+	int32_t listfivemins, listhours, listdays, listmonths, listyears, listtop, listjsonxml;
 } CFG;
 
 /* internal interface information structure */
 typedef struct {
 	char name[32];
-	int filled;
+	short filled;
+	short is64bit;
 	uint64_t rx;
 	uint64_t tx;
 	uint64_t rxp;
 	uint64_t txp;
+	time_t timestamp;
 } IFINFO;
 
-typedef struct {
-	time_t date;
-	uint64_t rx, tx;
-} HOUR;
-
-typedef struct {
-	time_t date;
-	uint64_t rx, tx;
-	int rxk, txk;
-	int used;
-} DAY;
-
-typedef struct {
-	time_t month;
-	uint64_t rx, tx;
-	int rxk, txk;
-	int used;
-} MONTH;
-
-/* db structure */
-typedef struct {
-	int version;
-	char interface[32];
-	char nick[32];
-	int active;
-	uint64_t totalrx, totaltx, currx, curtx;
-	int totalrxk, totaltxk;
-	time_t lastupdated, created;
-	DAY day[30];
-	MONTH month[12];
-	DAY top10[10];
-	HOUR hour[24];
-	uint64_t btime;
-} DATA;
-
 typedef struct ibwnode {
 	char interface[32];
 	uint32_t limit;
@@ -307,6 +295,7 @@ typedef struct ibwnode {
 
 typedef enum PrintType {
 	PT_Info = 0,
+	PT_Infoless,
 	PT_Error,
 	PT_Config,
 	PT_Multiline,
@@ -314,27 +303,26 @@ typedef enum PrintType {
 } PrintType;
 
 /* common functions */
-int printe(PrintType type);
-int logprint(PrintType type);
+int printe(const PrintType type);
+int logprint(const PrintType type);
 int verifylogaccess(void);
-int dmonth(int month);
-time_t mosecs(void);
-uint64_t countercalc(const uint64_t *a, const uint64_t *b);
-void addtraffic(uint64_t *destmb, int *destkb, const uint64_t srcmb, const int srckb);
-uint64_t mbkbtokb(uint64_t mb, uint64_t kb);
+int dmonth(const int month);
+int isleapyear(const int year);
+time_t mosecs(time_t month, time_t updated);
+uint64_t countercalc(const uint64_t *a, const uint64_t *b, const short is64bit);
 char *strncpy_nt(char *dest, const char *src, size_t n);
 int isnumeric(const char *s);
 void panicexit(const char *sourcefile, const int sourceline) __attribute__((noreturn));
 char *getversion(void);
+void timeused(const char *func, const int reset);
 
 /* global variables */
-extern DATA data;
 extern CFG cfg;
 extern IFINFO ifinfo;
-extern char errorstring[512];
+extern char errorstring[1024];
 extern ibwnode *ifacebw;
 extern int debug;
-extern int noexit;      /* = running as daemon if 2 */
+extern int noexit; /* = running as daemon if 2 */
 extern int intsignal;
 extern int pidfile;
 extern int disableprints;
